/*
 *  mplp.cpp
 *  dtrw
 *
 *  Created by Dhruv Batra on 1/22/11.
 *  Copyright 2011 Dhruv Batra. All rights reserved.
 *
 */

#include "mplp.h"
#include "constants_declaration.h"

using namespace std;

void MPLPwrapper(int nnodes, int nedges, int *cardinalities, int* edgelist, double* D, double* V)
{
    
    // MPLP code expects edge-energies in row-major format. V holds thems in column-major. So transpose.
    bool transpose = true;
    
    ////////////////////////////////////////////////////////////////////////////////
    // Initialize MPLP (Based on type of problem)
#ifdef TYPE_POTTS
    // Potts-Model implies that all nodes have same cardinality. Check that.
    int nlabels = cardinalities[0];
    for (int i=1; i!=nnodes; ++i)
        CHECK((nlabels==cardinalities[i]), "For Potts Model all nodes must have equal cardinalities");
    
    vector<double> potts_lambda(V,V+nedges);
    MPLPAlg mplp(nnodes, nedges, nlabels, edgelist, potts_lambda, D);
#else
    MPLPAlg mplp(nnodes, nedges, cardinalities, edgelist, V, D, transpose);
#endif
    
    ////////////////////////////////////////////////////////////////////////////////
    // Run MPLP
	mplp.RunMPLP(MAXITER,DUAL_IMPROV_THRES,INT_GAP_THRES);

    ////////////////////////////////////////////////////////////////////////////////
    // Write data to files
	vector<double> disthist(mplp.m_objhist.size(),nedges*2);
	ostringstream o; o << "_alg_t" << -2 << ".txt";
    
	string lbn = "lbhist", enn = "enhist", tn = "timehist";
	lbn = outputpath + "/" + lbn + o.str(); enn = outputpath + "/" + enn + o.str(); 
    tn = outputpath + "/" + tn + o.str(); 
	
	mplp.Write(NULL,NULL,NULL,(char*) lbn.c_str(),(char*) enn.c_str(),(char*) tn.c_str());
	
	WriteToFile(outputpath+"disthist"+o.str(), disthist);
    WriteToFile(outputpath+"primalsol"+o.str(), mplp.m_decoded_res);
}


void testMPLPGridPotts(int sizeX, int sizeY, int _numpairs, int K, double *D, double *V)
{
	////////////////////////////////////
	// parameters
	int MAXITER = 300;
	double obj_del_thr = 1e-5;
	double int_gap_thr = 1e-5;
	
	////////////////////////////////////
	// prepare input for mplp
	int n_nodes = sizeX*sizeY;
	vector<int> n_labels(n_nodes, K);
	vector<double> potts_lambda(V, V + _numpairs); // hope the pointer arithmetic works out
	
	////////////////////////////////////
	printf("nIters=%d\nobj_del_thr=%lg\nint_gap_thr=%lg\n",MAXITER,obj_del_thr,int_gap_thr);
	// initialize
	MPLPAlg mplp(sizeY, sizeX, n_labels, potts_lambda, D);
	// Run message passing
	mplp.RunMPLP(MAXITER,obj_del_thr,int_gap_thr);
	
	////////////////////////////////////
	// Save data to files
	vector<double> disthist(mplp.m_objhist.size(),_numpairs*2);
	
	ostringstream o; o << "_alg_t" << -2 << "_seed_" << 0 << ".txt";
	string lbn = "lbhist", enn = "enhist", tn = "timehist", dtn = "disthist";//, ern = "edgecnthist", tbn = "tbhist";
	lbn = name + "/" + lbn + o.str(); enn = name + "/" + enn + o.str(); tn = name + "/" + tn + o.str(); 
	dtn = name + "/" + dtn + o.str();
	
	mplp.Write(NULL,NULL,NULL,(char*) lbn.c_str(),(char*) enn.c_str(),(char*) tn.c_str());
	
	ofstream dtf; dtf.open(dtn.c_str()); dtf << disthist; dtf.close();
}
